/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __FLink
#define __FLink

#include "common.h"
#include "listState.h"
#include "fTcpSender.h"
#include "fSimOutput.h"
#include "fQueue.h"
#include <map>
#include <sstream>

typedef struct
{
  class ListState flow_rate;
  class ListState queue_delay;
  class ListState noloss_prob;
} FlowState;

class FLink
{
  std::list < class FTcpSender * >flows;
  std::map < FTcpSender *, FlowState > flow_states;

  std::map < FTcpSender *, double >sum_pkt_all_flow;
    std::map < FTcpSender *, double >pre_time_all_flow;

  FNode *prev, *next;

  class FQueue *queue;

  double delay;
  double BW;
  double link_prop_delay;

  const int prev_node_id;
  const int next_node_id;

  class FNode *prev_node;
  class FNode *next_node;
  class FSimOutput *output;

public:
    FLink (int prev_node_id_, int next_node_id_):prev_node_id (prev_node_id_),
    next_node_id (next_node_id_), queue (NULL)
  {
  }
   ~FLink ()
  {
  }
  void set_prev_node (FNode * prev_node_)
  {
    prev_node = prev_node_;
  }
  void set_next_node (FNode * next_node_)
  {
    next_node = next_node_;
  }
  void set_BW (double BW_)
  {
    BW = BW_;
  }
  void set_queue (FQueue * queue_)
  {
    queue = queue_;
  }
  void set_output (FSimOutput * output_)
  {
    output = output_;
  }
  int get_prev_node_id ()
  {
    return prev_node_id;
  }
  int get_next_node_id ()
  {
    return next_node_id;
  }
  double get_BW ()
  {
    return BW;
  }
  std::list < class FTcpSender * >*get_flows ()
  {
    return &flows;
  }

  std::string get_link_id ()
  {
    std::stringstream ss;
    ss << "[(" << prev_node_id << ")->(" << next_node_id << ")]";
    return ss.str ();
  }
  FQueue *get_queue ()
  {
    return queue;
  }
  double get_total_rate (double time)
  {
    double total_rate = 0.0;
    for (std::map < FTcpSender *, FlowState >::iterator it =
	 flow_states.begin (); it != flow_states.end (); it++)
      total_rate += ((*it).second).flow_rate.get_state (time);
    return total_rate;
  }
  void set_link_prop_delay (double link_prop_delay_)
  {
    assert (link_prop_delay >= 0);
    link_prop_delay = link_prop_delay_;
  }
  void init_link ();
  double get_link_prop_delay ()
  {
    return link_prop_delay;
  }
  bool match (int prev_node_id_, int next_node_id_)
  {
    if ((prev_node_id == prev_node_id_) && (next_node_id == next_node_id_))
      return true;
    else
      return false;
  }
  void add_flow (FTcpSender * flow);
  double get_rate (double time, FTcpSender * flow)
  {
    return flow_states[flow].flow_rate.get_state (time);
  }
  double get_queue_delay (double time, FTcpSender * flow)
  {
    return flow_states[flow].queue_delay.get_state (time);
  }
  double get_noloss_prob (double time, FTcpSender * flow)
  {
    return flow_states[flow].noloss_prob.get_state (time);
  }
  void get_state (double time, FTcpSender * flow, double *rate, double *delay,
		  double *noloss)
  {
    FlowState state = flow_states[flow];
    *rate = state.flow_rate.get_state (time);
    *delay = state.queue_delay.get_state (time);
    *noloss = state.noloss_prob.get_state (time);
  }

  void put_state (FTcpSender * flow, double time, double rate, double delay =
		  0.0, double noloss = 1.0, int prev_prev_node_id = 0);
  void print ();
  void print_trace_all_flow (FTcpSender * flow, int prev_prev_node_id,
			     double time, double rate, double delay,
			     double noloss);
  void print_namtrace_all_flow (FTcpSender * flow, int prev_prev_node_id,
				double time, double rate, double delay,
				double noloss);
  void print_stat ();
};

#endif
