/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __FRouter
#define __FRouter

#include "common.h"
#include "fNode.h"
#include "fLink.h"
#include "fTcpSender.h"
#include "fSimOutput.h"
#include "fQueue.h"
#include <map>

class LinksRate
{
  std::map < class FLink *, double >links_rate;

public:
    LinksRate ()
  {
  }
   ~LinksRate ()
  {
  }
  void init_zero ()
  {
    for (std::map < FLink *, double >::iterator it = links_rate.begin ();
	 it != links_rate.end (); it++)
      {
	it->second = 0.0;
      }
  }

  void add_link (FLink * const link_)
  {
    links_rate[link_] = 0.0;
  }

  void add_rate (FLink * const link_, const double rate)
  {
    if (links_rate.find (link_) != links_rate.end ())
      links_rate[link_] += rate;
  }
  double get_rate (FLink * const link_)
  {
    if (links_rate.find (link_) != links_rate.end ())
      return links_rate[link_];
    else
      return 0.0;
  }
};

typedef struct
{
  class FTcpSender *flow;
  FLink *input_link;
  FLink *output_link;
} Route;


class FRouter:public FNode
{
  // identification parameter
  const int router_id;
  // routing parameter
    std::list < Route > route_table;

  // OutputList output_links;
  LinksRate output_links_arrival_rate;

  // int input_queue_eq_num;

  // other parameter
  class FSimOutput *output;
    std::list < FLink * >out_links;

public:
    FRouter (const int router_id_):router_id (router_id_)
  {
  }
   ~FRouter ()
  {
  }

  void get_all_queue_derivs (const double time, double *const derivs,
			     int *const ins, double *const state);
  void put_state (const double time, int *const ins, double *const state);
  void set_output (FSimOutput * const output_)
  {
    output = output_;
  }
  void add_route (FTcpSender * const flow, FLink * const prev_link,
		  FLink * const next_link);
  int get_eq_num ();
  int get_router_id ()
  {
    return router_id;
  }
  void print_stat ();
  void print_avg ();
  void set_max_min (int *ins, double *max_values, double *min_values);
};

#endif
