/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __FSIM
#define __FSIM

#include "common.h"
#include "fSimInput.h"
#include <vector>
#include <string>
#include <fstream>
#include "fSimOutput.h"

class FSimEngine
{
  FlowList flow_list;
  LinkList link_list;
  RouterList router_list;

  AvgData step_time;

  double *max_values;
  double *min_values;

  double *ytemp;
  double *dysav;
  double *dysav_mid;
  double *ysav;
  double *derivs1;
  double *derivs2;
  double *derivs3;
  double *state1;
  double *state2;
  double *state3;

  double sim_time;

  double init_delta;
  double init_thr;
  double RTT;
  double MINSTEP, MAXSTEP, TINY, PGROW, PSHRNK, FCOR, SAFETY, ERRCON, EPS;
  int flow_num;
  int router_num;
  int eq_num;

  class FSimOutput *output;
  bool print_flag_dyna, print_flag_avg, print_flag_trace_all,
    print_flag_namtrace_all;

public:
    FSimEngine ():max_values (NULL), min_values (NULL),
    print_flag_dyna (false), print_flag_avg (false),
    print_flag_trace_all (false), print_flag_namtrace_all (false)
  {
  }
   ~FSimEngine ();

  void read_input (char *setting_file_name, char *topology_file_name);
  void set_output (char *output_file_name, char *output_format_name);
  void init_event (double delta);
  void simulation ();
  void init (double *state);
  void rk4_all_node (double *state, double *derivs, double now, double delta,
		     double *state_out);
  void rkqs (double *state, double *derivs, double now, double init_delta,
	     double *actual_delta, double *next_time, double *next_delta,
	     double *scal);
  void get_derivs_all_node (double time, double *state, double *derivs);
  void set_max_min ();
  void print_nam_header (std::ofstream & out);
};

#endif
