/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "fSimInput.h"

FTcpSender *
FlowList::add_list (int src_node_id, int dst_node_id)
{
  for (std::list < FTcpSender * >::iterator it = flow_list.begin ();
       it != flow_list.end (); it++)
    {
      if ((*it)->match (src_node_id, dst_node_id) == true)
	{
	  (*it)->inc_flow_num ();
	  return NULL;
	}
    }

  FTcpSender *new_flow = new FTcpSender ();
  new_flow->set_src_node_id (src_node_id);
  new_flow->set_dst_node_id (dst_node_id);
  new_flow->set_flow_num (1);
  flow_list.push_back (new_flow);

  return new_flow;

}

void
RouterList::print_nam_header (std::ofstream & out)
{
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      out << "n -t * -a " << (*it)->get_router_id () << " -s " << (*it)->
	get_router_id () << " -S UP -v circle -c black -i black\n";
    }
}
void
FlowList::get_derivs (double time, double *state, double *derivs)
{
  int i = eq_offset;

  for (std::list < FTcpSender * >::iterator it = flow_list.begin ();
       it != flow_list.end (); it++)
    {
      derivs[i] = (*it)->thr_derivs (time, state[i]);
      i++;
    }
}

void
FlowList::flow_puts (double time, double *state)
{
  int i = eq_offset;
  for (std::list < FTcpSender * >::iterator it = flow_list.begin ();
       it != flow_list.end (); it++)
    {
      (*it)->puts_state (time, state[i]);
      i++;
    }
}

void
FlowList::print ()
{
  for (std::list < FTcpSender * >::iterator it = flow_list.begin ();
       it != flow_list.end (); it++)
    {
      (*it)->print_stat ();
    }
}
void
FlowList::print_avg ()
{
  for (std::list < FTcpSender * >::iterator it = flow_list.begin ();
       it != flow_list.end (); it++)
    {
      (*it)->print_avg ();
    }
}
void
FlowList::set_max_min (double *max_values, double *min_values)
{
  int i = eq_offset;
  for (std::list < FTcpSender * >::iterator it = flow_list.begin ();
       it != flow_list.end (); it++)
    {
      (*it)->set_max_min (&i, max_values, min_values);
    }
}

void
LinkList::init_links ()
{
  for (std::list < FLink * >::iterator it = link_list.begin ();
       it != link_list.end (); it++)
    {
      (*it)->init_link ();
    }
}
FLink *
LinkList::add_list (int prev_node_id, int next_node_id, FTcpSender * flow)
{
  if (prev_node_id != next_node_id)
    {
      for (std::list < FLink * >::iterator it = link_list.begin ();
	   it != link_list.end (); it++)
	{
	  if ((*it)->match (prev_node_id, next_node_id) == true)
	    {
	      (*it)->add_flow (flow);
	      return (*it);
	    }
	}
    }

  FLink *new_link = new FLink (prev_node_id, next_node_id);
  new_link->add_flow (flow);
  link_list.push_back (new_link);
  return new_link;

}

void
LinkList::print_nam_header (std::ofstream & out)
{
  for (std::list < FLink * >::iterator it = link_list.begin ();
       it != link_list.end (); it++)
    {
      if ((*it)->get_prev_node_id () * (*it)->get_next_node_id () >= 0)
	{
	  out << "l -t * -s " << (*it)->
	    get_prev_node_id () << " -d " << (*it)->
	    get_next_node_id () << " -S -r " << (*it)->get_BW () *
	    1000000 << " -D 0.01 -c black\n";
	}
    }
}

void
LinkList::print ()
{
  for (std::list < FLink * >::iterator it = link_list.begin ();
       it != link_list.end (); it++)
    {
      (*it)->print_stat ();
    }
}

FRouter *
RouterList::add_list (int router_id, FLink * prev_link, FLink * next_link,
		      FTcpSender * flow)
{
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      if ((*it)->get_router_id () == router_id)
	{
	  (*it)->add_route (flow, prev_link, next_link);
	  return (*it);
	}
    }

  FRouter *new_router = new FRouter (router_id);
  new_router->add_route (flow, prev_link, next_link);

  router_list.push_back (new_router);
  return new_router;
}

void
RouterList::get_derivs (double time, double *state, double *derivs)
{
  int i = eq_offset;
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      (*it)->get_all_queue_derivs (time, derivs, &i, state);
    }
}

void
RouterList::router_puts (double time, double *state)
{
  int i = eq_offset;
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      (*it)->put_state (time, &i, state);
    }
}

void
RouterList::print ()
{
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      (*it)->print_stat ();
    }
}

void
RouterList::print_avg ()
{
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      (*it)->print_avg ();
    }
}
void
RouterList::set_max_min (double *max_values, double *min_values)
{
  int i = eq_offset;
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      (*it)->set_max_min (&i, max_values, min_values);
    }
}

int
RouterList::get_eq_num ()
{
  int eq_num_ = 0;
  for (std::list < FRouter * >::iterator it = router_list.begin ();
       it != router_list.end (); it++)
    {
      eq_num_ += (*it)->get_eq_num ();
    }
  return eq_num_;
}
