/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __FSIMINPUT
#define __FSIMINPUT

#include <string>
#include <list>
#include "fTcpSender.h"
#include "fRouter.h"
#include "fQueue.h"
#include "fLink.h"

class FlowList
{
  std::list < FTcpSender * >flow_list;
  int eq_offset;
public:
    FlowList ():eq_offset (0)
  {
  }
  FTcpSender *add_list (int stc_node_id, int dst_node_id);
  void get_derivs (double time, double *state, double *derivs);
  void flow_puts (double time, double *state);
  void set_eq_offset (int eq_offset_)
  {
    eq_offset = eq_offset_;
  }
  int size ()
  {
    return flow_list.size ();
  }
  void set_max_min (double *max_values, double *min_values);
  void print ();
  void print_avg ();
};

class LinkList
{
  std::list < FLink * >link_list;
public:
  FLink * add_list (int prev_node_id, int next_node_id, FTcpSender * flow);

  void print ();
  void init_links ();
  void print_nam_header (std::ofstream & out);
  int size ()
  {
    return link_list.size ();
  }
};

class RouterList
{
  std::list < FRouter * >router_list;
  int eq_offset;
public:
    RouterList ():eq_offset (0)
  {
  }

  FRouter *add_list (int router_id, FLink * prev_link, FLink * next_link,
		     FTcpSender * flow);
  std::list < FRouter * >*get_router_list ()
  {
    return &router_list;
  }
  void get_derivs (double time, double *state, double *derivs);
  void router_puts (double time, double *state);
  void set_eq_offset (int eq_offset_)
  {
    eq_offset = eq_offset_;
  }
  int size ()
  {
    return router_list.size ();
  }
  void set_max_min (double *max_values, double *min_values);
  int get_eq_num ();
  void print_nam_header (std::ofstream & out);
  void print_avg ();
  void print ();
};

#endif
