/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __FTcpSender
#define __FTcpSender

#include "common.h"
#include "listState.h"
#include "fNode.h"
#include "fLink.h"
#include "fSimOutput.h"
#include "fRouter.h"
#include "fQueue.h"
#include <sstream>

class FTcpSender:public FNode
{
  ListState thr;
  ListState rtt;
  double loss;

  AvgData avg_thr, avg_rtt, avg_loss;

  int port_id;
  double prop_delay;
  double init_thr;
  double max_window_size;
  double max_throughput;

  double init_rtt;

  int flow_ID;
  int flow_num;
  int src_node_id;
  int src_port_id;
  int dst_node_id;
  int dst_port_id;
  int next_node_id;

  class FLink *send_link;
  class FLink *sink_link;

  class FSimOutput *output;

    std::list < class FQueue * >queue_list;

public:
    FTcpSender ();
  void print (double now);
  void print_stat ();

  void set_src_port_id (int src_port_id_)
  {
    src_port_id = src_port_id_;
  }
  void set_dst_port_id (int dst_port_id_)
  {
    dst_port_id = dst_port_id_;
  }
  void set_src_node_id (int src_node_id_)
  {
    src_node_id = src_node_id_;
  }
  void set_dst_node_id (int dst_node_id_)
  {
    dst_node_id = dst_node_id_;
  }
  void set_max_window_size (double max_window_size_)
  {
    max_window_size = max_window_size_;
  }
  void set_output (FSimOutput * output_)
  {
    output = output_;
  }
  void set_send_link (FLink * send_link_)
  {
    send_link = send_link_;
  }
  void set_sink_link (FLink * sink_link_)
  {
    sink_link = sink_link_;
  }
  void set_flow_ID (int flow_ID_)
  {
    flow_ID = flow_ID_;
  }
  void set_flow_num (int flow_num_)
  {
    flow_num = flow_num_;
  }
  void set_next_node_id (int next_node_id_)
  {
    next_node_id = next_node_id_;
  }
  void add_queue (FQueue * queue_)
  {
    queue_list.push_back (queue_);
  }
  double calc_rtt (double time);
  double calc_loss (double time);

  void set_max_min (int *ins, double *max_values, double *min_values)
  {
    max_values[*ins] = max_window_size / init_rtt;
    min_values[*ins] = init_thr;
    *ins += 1;
  }
  double get_noloss_in_route ()
  {
    return (1.0 - loss);
  }
  std::string get_flow_id ()
  {
    std::stringstream ss;
    ss << "[(" << src_node_id << "," << src_port_id
      << ")->(" << dst_node_id << "," << dst_port_id << ")]";
    return ss.str ();
  }
  int get_next_node_id ()
  {
    return next_node_id;
  }
  int get_flow_ID ()
  {
    return flow_ID;
  }
  double get_rtt (double time)
  {
    double rtt_ = rtt.get_state (time);
    if (rtt_ <= init_rtt)
      return init_rtt;
    return rtt_;
  }
  void set_init_rtt (double init_rtt_)
  {
    init_rtt = init_rtt_;
  }
  void set_prop_delay (double prop_delay_)
  {
    prop_delay = prop_delay_;
    thr.set_timeout (prop_delay);
    rtt.set_timeout (prop_delay);
  }
  void set_init_thr (double init_thr_)
  {
    init_thr = init_thr_;
    thr.put_state (0.0, init_thr);
  }
  void inc_flow_num ()
  {
    flow_num++;
  }

  int get_src_node_id ()
  {
    return src_node_id;
  }
  int get_dst_node_id ()
  {
    return dst_node_id;
  }
  int get_src_port_id ()
  {
    return src_port_id;
  }
  int get_dst_port_id ()
  {
    return dst_port_id;
  }

  double thr_derivs (double time, double thr_now);

  void puts_state (double time, double thr);
  FLink *get_send_link ()
  {
    return send_link;
  }
  FLink *get_sink_link ()
  {
    return sink_link;
  }

  int get_flow_num ()
  {
    return flow_num;
  }

  bool match (int src_node_id_, int dst_node_id_)
  {
    if ((src_node_id == src_node_id_) && (dst_node_id == dst_node_id_))
      return true;
    else
      return false;
  }
  void print ();
  void print_avg ();
};


#endif
