/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "listState.h"
#include <sstream>

void
ListState::push_back (double time, double data)
{
  SNode *new_state = new SNode;
  new_state->state.time = time;
  new_state->state.data = data;
  new_state->next = NULL;
  if (head == NULL && tail == NULL)
    {
      head = tail = new_state;
    }
  else
    {
      tail->next = new_state;
      tail = new_state;
    }
}
void
ListState::pop_front ()
{
  SNode *tmp;
  if (head == NULL)
    return;
  tmp = head;
  if (head->next == NULL)
    {
      head = tail = NULL;
    }
  else
    {
      head = head->next;
    }
  delete tmp;
}

void
ListState::put_state (double time, double data)
{
  double end_time;
  if (maxStateTime > time)
    return;
  if (maxStateTime < time)
    maxStateTime = time;

  if (maxStateTime == time && tail != NULL)
    tail->state.data = data;
  else
    this->push_back (time, data);
}

double
ListState::get_state (double time)
{
  SNode *p;
  SNode *q;

  if (head == NULL)
    return 0.0;

  while (maxStateTime - head->state.time > timeout_time)
    {
      if (head->next == NULL)
	break;
      if (head->next->next == NULL)
	break;
      this->pop_front ();
    }

  if (time < head->state.time)
    {
      return 0.0;
    }

  if (head->next == NULL)
    return head->state.data;

  if (maxStateTime - time < 0.001 || maxStateTime < time)
    {
      return tail->state.data;
    }

  p = head;
  q = p->next;
  while (q != NULL)
    {
      // assume min_step < 0.0001 
      if (q->state.time - p->state.time < 0.001)
	{
	  return p->state.data;
	}
      if (q->state.time > time)
	{
	  double dat = (time - p->state.time)
	    * (q->state.data - p->state.data) / (q->state.time -
						 p->state.time) +
	    p->state.data;
	  return dat;
	}
      p = q;
      q = q->next;
    }
  return p->state.data;
}

void
ListState::test ()
{
  set_timeout (5.0);

  for (int i = 0; i < 20; i++)
    {
      put_state (i * 5, i * i);
    }

  std::cout << print ();
  std::cout << "-1 " << get_state (-1) << std::endl;
  std::cout << print ();
  std::cout << "51 " << get_state (51) << std::endl;
  std::cout << print ();
  std::cout << "60 " << get_state (60) << std::endl;
  std::cout << print ();
  std::cout << "79 " << get_state (79) << std::endl;
  std::cout << print ();
  put_state (120, 400);
  std::cout << "91 " << get_state (91) << std::endl;
  std::cout << print ();
  std::cout << "111 " << get_state (111) << std::endl;
  std::cout << print ();
}

std::string ListState::print ()
{
  std::stringstream ss;
  SNode *
    p = head;
  while (p != NULL)
    {
      ss << "\t time : " << p->state.time << " data : " << p->state.
	data << std::endl;
      p = p->next;
    }
  ss << std::endl;

  return ss.str ();
}
