/*

  Copyright (C) 2007, Yusuke Sakumoto.
  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "common.h"
#include "fSimEngine.h"
#include "fSimOutput.h"
#include <ctime>
#include <cstdlib>
#include <cstring>

#define MAX_FILE_NAME 256
#define DEF_OUTPUT "out.tr"
#define DEF_CONF "system.conf"
#define DEF_FORMAT "fsim"

void
usage ()
{
  std::cerr << "Usage:  fsim [options] topology_file\n"
    <<
    "  -c <conf_file>     : Load in simulator configuration from <conf_file>\n"
    <<
    "                       If none specified this option, FSIM load simulator configuration from 'system.conf'\n"
    << "  -o <output_file>   : Write simulation results to <output_file>.\n"
    <<
    "                       If none specified this option , FSIM writes results to 'out.tr'.\n"
    <<
    "  -n                 : Use this option when you want to input topology files compatible with ns-2.\n"
    <<
    "  -f <output_format> : Specify output format <output_format> of simulation results.\n"
    << "\n" << "          OUTPUT FORMAT LIST\n" <<
    "             fsim [default] : Write dynamics values of network states\n"
    <<
    "             fsim_avg       : Write average values of network states in the simulation\n"
    <<
    "             trace_all      : Write trace data like results of trace_all command\n"
    <<
    "             namtrace_all   : Write trace data like results of namtrace_all command\n"
    << "\n" << "  -h                 : display this help and exit\n";
  std::exit (1);
}

int
main (int argc, char **argv)
{
  FSimEngine fsim;
  char output_file_name[MAX_FILE_NAME];
  char configure_file_name[MAX_FILE_NAME];
  char output_format[MAX_FILE_NAME];
  char topology_file_name[MAX_FILE_NAME];
  char tmp_file_name[MAX_FILE_NAME];
  char command[MAX_FILE_NAME * 2];
  char opt;
  bool ns_flag = false;
  extern char *optarg;
  extern int optind, opterr;

  std::strcpy (output_file_name, DEF_OUTPUT);
  std::strcpy (configure_file_name, DEF_CONF);
  std::strcpy (output_format, DEF_FORMAT);

  while ((opt = getopt (argc, argv, "c:no:f:h")) != EOF)
    {
      switch (opt)
	{
	case 'c':
	  std::strcpy (configure_file_name, optarg);
	  break;
	case 'n':
	  ns_flag = true;
	  break;
	case 'o':
	  std::strcpy (output_file_name, optarg);
	  break;
	case 'f':
	  std::strcpy (output_format, optarg);
	  break;
	case 'h':
	default:
	  usage ();
	}
    }

  argc -= optind;
  argv += optind;

  if (argc < 1)
    {
      usage ();
    }
  if (ns_flag)
    {
      std::sprintf (topology_file_name, "/tmp/fsim%d.tmp", getpid ());
      std::sprintf (command, "tcl2tpl %s > %s", argv[0], topology_file_name);
      std::system (command);
    }
  else
    {
      std::strcpy (topology_file_name, argv[0]);
    }

  std::srand ((unsigned) std::time (NULL));

  fsim.set_output (output_file_name, output_format);
  fsim.read_input (configure_file_name, topology_file_name);
  fsim.simulation ();

  if (!strcmp (output_format, "trace_all"))
    {
      std::sprintf (command, "sort -n +1 -o %s %s", output_file_name,
		    output_file_name);
      std::system (command);
    }
  else if (!strcmp (output_format, "namtrace_all"))
    {
      char tmp_file_name[MAX_FILE_NAME];
      char tmp_file_name2[MAX_FILE_NAME];
      std::sprintf (tmp_file_name, "/tmp/fsim_nam%d.tmp", getpid ());
      std::ofstream tmp_out (tmp_file_name);
      fsim.print_nam_header (tmp_out);
      tmp_out.close ();
      std::sprintf (tmp_file_name2, "/tmp/fsim_nam2%d.tmp", getpid ());
      std::sprintf (command, "sort -ns +2 -3 -o %s %s; cat %s %s > %s",
		    tmp_file_name2, output_file_name, tmp_file_name,
		    tmp_file_name2, output_file_name);
      std::system (command);
    }
  return 0;
}
